@@ -8,19 +8,14 @@ import re
 import sys
 import types
 import unittest
-from types import ClassType, TypeType
+from nose.pyversion import ClassType, TypeType, isgenerator, ismethod
+
 
-try:
-    from compiler.consts import CO_GENERATOR
-except ImportError:
-    # IronPython doesn't have a complier module
-    CO_GENERATOR=0x20
-    
 log = logging.getLogger('nose')
 
 ident_re = re.compile(r'^[A-Za-z_][A-Za-z0-9_.]*$')
 class_types = (ClassType, TypeType)
-skip_pattern = r"(?:\.svn)|(?:[^.]+\.py[co])|(?:.*~)|(?:.*\$py\.class)"
+skip_pattern = r"(?:\.svn)|(?:[^.]+\.py[co])|(?:.*~)|(?:.*\$py\.class)|(?:__pycache__)"
 
 try:
     set()
@@ -147,19 +142,6 @@ def file_like(name):
             or not ident_re.match(os.path.splitext(name)[0]))
 
 
-def cmp_lineno(a, b):
-    """Compare functions by their line numbers.
-
-    >>> cmp_lineno(isgenerator, ispackage)
-    -1
-    >>> cmp_lineno(ispackage, isgenerator)
-    1
-    >>> cmp_lineno(isgenerator, isgenerator)
-    0
-    """
-    return cmp(func_lineno(a), func_lineno(b))
-
-
 def func_lineno(func):
     """Get the line number of a function. First looks for
     compat_co_firstlineno, then func_code.co_first_lineno.
@@ -181,11 +163,6 @@ def isclass(obj):
     return obj_type in class_types or issubclass(obj_type, type)
 
 
-def isgenerator(func):
-    try:
-        return func.func_code.co_flags & CO_GENERATOR != 0
-    except AttributeError:
-        return False
 # backwards compat (issue #64)
 is_generator = isgenerator
 
@@ -283,7 +260,7 @@ def getpackage(filename):
     'nose.plugins'
     """
     src_file = src(filename)
-    if not src_file.endswith('.py') and not ispackage(src_file):
+    if (os.path.isdir(src_file) or not src_file.endswith('.py')) and not ispackage(src_file):
         return None
     base, ext = os.path.splitext(os.path.basename(src_file))
     if base == '__init__':
@@ -308,7 +285,7 @@ def ln(label):
     '---------------------------- hello there -----------------------------'
     """
     label_len = len(label) + 2
-    chunk = (70 - label_len) / 2
+    chunk = (70 - label_len) // 2
     out = '%s %s %s' % ('-' * chunk, label, '-' * chunk)
     pad = 70 - len(out)
     if pad > 0:
@@ -387,7 +364,7 @@ def split_test_name(test):
                 # nonsense like foo:bar:baz
                 raise ValueError("Test name '%s' could not be parsed. Please "
                                  "format test names as path:callable or "
-                                 "module:callable.")
+                                 "module:callable." % (test,))
     elif not tail:
         # this is a case like 'foo:bar/'
         # : must be part of the file path, so ignore it
@@ -431,8 +408,6 @@ def test_address(test):
                 file = os.path.abspath(file)
         call = getattr(test, '__name__', None)
         return (src(file), module, call)
-    if t == types.InstanceType:
-        return test_address(test.__class__)
     if t == types.MethodType:
         cls_adr = test_address(test.im_class)
         return (src(cls_adr[0]), cls_adr[1],
@@ -455,6 +430,9 @@ def test_address(test):
             method_name = test._testMethodName
         return (src(cls_adr[0]), cls_adr[1],
                 "%s.%s" % (cls_adr[2], method_name))
+    if (hasattr(test, '__class__') and
+            test.__class__.__module__ not in ('__builtin__', 'builtins')):
+        return test_address(test.__class__)
     raise TypeError("I don't know what %s is (%s)" % (test, t))
 test_address.__test__ = False # do not collect
 
@@ -469,9 +447,10 @@ def try_run(obj, names):
         if func is not None:
             if type(obj) == types.ModuleType:
                 # py.test compatibility
-                try:
-                    args, varargs, varkw, defaults = inspect.getargspec(func)
-                except TypeError:
+                if isinstance(func, types.FunctionType):
+                    args, varargs, varkw, defaults = \
+                        inspect.getargspec(func)
+                else:
                     # Not a function. If it's callable, call it anyway
                     if hasattr(func, '__call__'):
                         func = func.__call__
@@ -506,23 +485,24 @@ def src(filename):
     return filename
 
 
-def match_last(a, b, regex):
-    """Sort compare function that puts items that match a
+def regex_last_key(regex):
+    """Sort key function factory that puts items that match a
     regular expression last.
 
     >>> from nose.config import Config
+    >>> from nose.pyversion import sort_list
     >>> c = Config()
     >>> regex = c.testMatch
     >>> entries = ['.', '..', 'a_test', 'src', 'lib', 'test', 'foo.py']
-    >>> entries.sort(lambda a, b: match_last(a, b, regex))
+    >>> sort_list(entries, regex_last_key(regex))
     >>> entries
     ['.', '..', 'foo.py', 'lib', 'src', 'a_test', 'test']
     """
-    if regex.search(a) and not regex.search(b):
-        return 1
-    elif regex.search(b) and not regex.search(a):
-        return -1
-    return cmp(a, b)
+    def k(obj):
+        if regex.search(obj):
+            return (1, obj)
+        return (0, obj)
+    return k
 
 
 def tolist(val):
@@ -630,8 +610,13 @@ def transplant_func(func, module):
 
     """
     from nose.tools import make_decorator
-    def newfunc(*arg, **kw):
-        return func(*arg, **kw)
+    if isgenerator(func):
+        def newfunc(*arg, **kw):
+            for v in func(*arg, **kw):
+                yield v
+    else:
+        def newfunc(*arg, **kw):
+            return func(*arg, **kw)
 
     newfunc = make_decorator(func)(newfunc)
     newfunc.__module__ = module
